//+------------------------------------------------------------------+
//|                                                     averages.mq5 |
//+------------------------------------------------------------------+
#property copyright "mladen"
#property link      "mladenfx@gmail.com"
#property version   "1.00"

//
//
//
//
//

#property indicator_chart_window
#property indicator_buffers 2
#property indicator_plots   1

#property indicator_label1  "Label1"
#property indicator_type1   DRAW_COLOR_LINE
#property indicator_color1  DeepSkyBlue,PaleVioletRed
#property indicator_style1  STYLE_SOLID
#property indicator_width1  2

//
//
//
//
//

enum enumAveragesType
{
   avgSma,    // Simple moving average
   avgEma,    // Exponential moving average
   avgSmma,   // Smoothed moving average
   avgLwma,   // Linear weighted moving average
   avgLsma,   // Linear regression value
   avgTrima,  // Triangular moving average
   avgSwma,   // Sine weighted moving average
   avgHullma, // Hull moving average
   avgT3ma,   // T3 moving average
   avgQuant   // Qunatiles
};

input int              AveragePeriod = 14;    // Calculation period
input enumAveragesType AverageType   = 0;     // Calculation type
input string           _             = "";    // T3 parameters (used only if T3 chosen)
input double           T3Hot         = 0.7;   // T3 hot value
input bool             T3Original    = false; // T3 original Tillson calculation?
input string           __            = "";    // Quantile parameters (used only if Quantiles chosen)
input double           QuantPercent  = 50.0;  // Quantile percent (50 == Median)
input bool             Multicolor    = true;  // Multi color mode?

//
//
//
//
//
//

double MaBuffer[];
double ColorBuffer[];
enumAveragesType gAverageType;

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int OnInit()
{
   SetIndexBuffer(0,MaBuffer,INDICATOR_DATA);
   SetIndexBuffer(1,ColorBuffer,INDICATOR_COLOR_INDEX);
            gAverageType = AverageType;
   PlotIndexSetString(0,PLOT_LABEL,getAverageName(gAverageType)+" ("+(string)AveragePeriod+")");
   IndicatorSetString(INDICATOR_SHORTNAME,getAverageName(gAverageType)+" ("+(string)AveragePeriod+")");
   return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int OnCalculate (const int rates_total, const int prev_calculated, const int begin,  const double& price[] )
{
   //
   //
   //
   //
   //
      
   static averages caverage; 
      if (AverageType==avgT3ma  && !caverage.T3Initialized)    caverage.setT3Parameters(AveragePeriod,T3Hot,T3Original);
      if (AverageType==avgQuant && !caverage.QuantInitialized) caverage.setQuantParameters(QuantPercent);
      
      //
      //
      //
      //
      //
      
      for (int i=(int)MathMax(prev_calculated-1,0); i<rates_total; i++)
      {
         MaBuffer[i] = caverage.calc(NULL,0,AveragePeriod,gAverageType,price[i],i);
         if (Multicolor && i>0)
         {
            ColorBuffer[i] = ColorBuffer[i-1];
               if (MaBuffer[i]>MaBuffer[i-1]) ColorBuffer[i]=0;
               if (MaBuffer[i]<MaBuffer[i-1]) ColorBuffer[i]=1;
         }
         else ColorBuffer[i]=0;
      }
   
   //
   //
   //
   //
   //
   
   return(rates_total);
}


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

string methodNames[] = {"Simple MA","Eexponential MA","Smoothed MA","Liner weighted MA","Linear regression value","Triangular MA","Sine weighted MA","Hull MA","T3 MA","Quantile"};
string getAverageName(enumAveragesType& method)
{
   method=MathMax(MathMin(method,10),0); return(methodNames[((int)method)]);
}

//
//
//
//
//

class averages
{
   protected:
      double aResultBuffer[];
      double aPricesBuffer[];
      double t3c1,t3c2,t3c3,t3c4,t3period;
      bool   t3Original;
      double quantPercent;

   //
   //
   //
   //
   //
         
   public :
      bool   T3Initialized;
      bool   QuantInitialized;
      
      //
      //
      //
      //
      //
      
      void   averages() { T3Initialized=false; QuantInitialized=false; };
      double calc(string symbol, ENUM_TIMEFRAMES period, double cPeriod, int cType, double cPrice, int i);
      void   setT3Parameters(double cPeriod, double cT3Hot, bool cT3Original);
      void   setQuantParameters(double cPercent);
};

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

void averages::setQuantParameters(double cPercent)
{
   quantPercent = MathMax(MathMin(cPercent,100),0);
   QuantInitialized = true;            
}
void averages::setT3Parameters(double cPeriod, double cT3Hot, bool cT3Original)
{
   cT3Hot = MathMax(MathMin(cT3Hot,1),0.0001);
   double a    = cT3Hot;
          t3c1 = -a*a*a;
          t3c2 =  3*(a*a+a*a*a);
          t3c3 = -3*(2*a*a+a+a*a*a);
          t3c4 = 1+3*a+a*a*a+3*a*a;
          t3period  = cPeriod; 
            if (!cT3Original) t3period = 1.0 + (t3period-1.0)/2.0;
   T3Initialized = true;            
}



//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//


double averages::calc(string symbol, ENUM_TIMEFRAMES period, double cPeriod, int cType, double cPrice, int i)
{
   if (symbol==NULL) symbol = _Symbol;
   if (period==0)    period = _Period;
   int bars = Bars(symbol,period);
         if (ArrayRange(aPricesBuffer,0)!=bars) ArrayResize(aPricesBuffer,bars);
                        aPricesBuffer[i] = cPrice;
      
   //
   //
   //
   //
   //
   //
      
   switch (cType)
   {
      case avgSma:
      {
         double avg = 0;  int k;
               for (k=0; k<cPeriod && (i-k)>=0; k++ ) avg += aPricesBuffer[i-k];
         return(avg/k);
      }                  
         
      //
      //
      //  
      //
      //
         
      case avgEma:
      {
         if (ArrayRange(aResultBuffer,0)!=bars) ArrayResize(aResultBuffer,bars);
         if (i>0)
                aResultBuffer[i] = aResultBuffer[i-1]+(2/(cPeriod+1))*(cPrice-aResultBuffer[i-1]);
         else   aResultBuffer[i] = aPricesBuffer[i];
         return(aResultBuffer[i]);
      }
         
      //
      //
      //  
      //
      //
         
      case avgSmma:
      {
         if (ArrayRange(aResultBuffer,0)!=bars) ArrayResize(aResultBuffer,bars);
         if (i<=cPeriod)
         {
            double avg = 0;  int k;
            for (k=0; k<cPeriod && (i-k)>=0; k++ ) avg += aPricesBuffer[i-k];
            if (k>0)                                                         
                  aResultBuffer[i] = avg/k;
            else  aResultBuffer[i] = aPricesBuffer[i];
         }
         else   aResultBuffer[i] = (aResultBuffer[i-1]*(cPeriod-1)+aPricesBuffer[i])/cPeriod;
         return(aResultBuffer[i]);
      }                  

      //
      //
      //  
      //
      //
         
      case avgLwma:
      {
         double sum  = 0;
         double sumw = 0;
            for (int k=0; k<cPeriod && (i-k)>=0; k++ )
            {
               double weight =  cPeriod-k;
                      sumw   += weight;
                      sum    += weight*aPricesBuffer[i-k];  
            }             
            if (sumw!=0)
                  return(sum/sumw);
            else  return(EMPTY_VALUE);
      }                  
      
      //
      //
      //
      //
      //
      
      case avgLsma:
      {
         static averages lwma;
         static averages sma;
            return (3*lwma.calc(symbol,period,cPeriod,avgLwma,cPrice,i)-2*sma.calc(symbol,period,cPeriod,avgSma,cPrice,i));
      }
      
      //
      //  
      //
      //
      //
         
      case avgTrima:
      {
         double half = (cPeriod+1.0)/2.0;
         double sum  = 0;
         double sumw = 0;
            for (int k=0; k<cPeriod && (i-k)>=0; k++ )
            {
               double weight =  cPeriod-k;  if (weight > half) weight = cPeriod-k;
                      sumw   += weight;
                      sum    += weight*aPricesBuffer[i-k];  
            }             
            if (sumw!=0)
                  return(sum/sumw);
            else  return(EMPTY_VALUE);
      }                  

      //
      //  
      //
      //
      //
         
      case avgSwma:
      {
         #define Pi 3.14159265358979323846
         double sum  = 0;
         double sumw = 0;
            for (int k=0; k<cPeriod && (i-k)>=0; k++ )
            {
               double weight = MathSin(Pi*(k+1)/(cPeriod+1));
                      sumw   += weight;
                      sum    += weight*aPricesBuffer[i-k];  
            }             
            if (sumw!=0)
                  return(sum/sumw);
            else  return(EMPTY_VALUE);
      }                  
      
      //
      //
      //
      //
      //

      case avgHullma:
      {
         int HalfPeriod = (int)MathFloor(cPeriod/2);
         int HullPeriod = (int)MathFloor(MathSqrt(cPeriod));
            static averages lwma1;
            static averages lwma2;
            static averages lwma3;
            double price1 = lwma1.calc(symbol,period,HalfPeriod,avgLwma,cPrice,i)*2-lwma2.calc(symbol,period,cPeriod,avgLwma,cPrice,i);
            return (lwma3.calc(symbol,period,HullPeriod,avgLwma,price1,i));
      }

      //
      //
      //
      //
      //
      
      case avgQuant:
      {
         static double quantileArray[];
         if (ArraySize(quantileArray)!=cPeriod) ArrayResize(quantileArray,(int)cPeriod);
                     for(int k=0; k<cPeriod && (i-k)>=0; k++) quantileArray[k] = aPricesBuffer[i-k];
         ArraySort(quantileArray);
      
         //
         //
         //
         //
         //
   
         double index = (cPeriod-1)*quantPercent/100.0;
         int    ind   = (int)index;
         double delta = index - ind;
         if (ind == NormalizeDouble(index,5))
               return(            quantileArray[ind]);
         else  return((1.0-delta)*quantileArray[ind]+delta*quantileArray[ind+1]);
      }                  

      //
      //
      //
      //
      //

      case avgT3ma:
      {
        
            static averages ema1; double price1 = ema1.calc(symbol,period,t3period,avgEma,cPrice,i);
            static averages ema2; double price2 = ema2.calc(symbol,period,t3period,avgEma,price1,i);
            static averages ema3; double price3 = ema3.calc(symbol,period,t3period,avgEma,price2,i);
            static averages ema4; double price4 = ema4.calc(symbol,period,t3period,avgEma,price3,i);
            static averages ema5; double price5 = ema5.calc(symbol,period,t3period,avgEma,price4,i);
            static averages ema6; double price6 = ema6.calc(symbol,period,t3period,avgEma,price5,i);
            
         return (t3c1*price6 + t3c2*price5 + t3c3*price4 + t3c4*price3);
      }
   }
   
   //
   //
   //
   //
   //
  
   return(EMPTY_VALUE);
}